/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

////////
// super-memory-miserly hash list storage (def.)
// Kris Popendorf
///////

#include "ecohash.h"
#include "ecolist.h"
#include "murasaki.h"

using namespace std;

EcoHash::EcoHash(BitSequence *pat) :
  Hash(pat),
  sorted(hash_size)
{
  fasta=new Ecolist[hash_size];
}

EcoHash::~EcoHash(){
  delete[] fasta;
}

void EcoHash::clear(){
  for(word i=0;i<hash_size;i++)
    fasta[i].clear();
  sorted.clear();
}

bool EcoHash::emptyAt(const HashKey key){
  return !fasta[key].getSize();
}

word EcoHash::sizeAt(const HashKey key){
  return fasta[key].getSize();
}

void EcoHash::add(const HashKey key,const HashVal &val){
  fasta[key].push(val.seqno,val.pos);
  sorted.unset(key);
}

void EcoHash::lookup(HashKey key,const HashVal &a,LocList &locList){
  if(!sorted.checkAndSet(key))
    fasta[key].sort();
  pair<Ecolist::iterator,Ecolist::iterator> range(fasta[key].equal_range(loc2val(a)));
  for(Ecolist::iterator i=range.first;i!=range.second;++i){
    val_type l(*i);
    locList[l.first].push_back(val2loc(l));
  }
}

void EcoHash::lookup(HashKey key,LocList &locList){
  Ecolist::iterator start(fasta[key].begin()),stop(fasta[key].end());
  for(Ecolist::iterator i=start;i!=stop;++i){
    val_type l(*i);
    locList[l.first].push_back(val2loc(l));
  }
}

bool EcoHash::lessthan(const val_type& a,const val_type& b){
  Location la(val2loc(a)),lb(val2loc(b));
  Window wina(lb,mfh->hashpat),winb(la,mfh->hashpat);
  return wina<winb;
}

void EcoHash::init(SeqPos _seq_count,word _longestSeq,word _totalHashLength,SeqPos _hash_bits){
  Ecolist::init(_seq_count,_longestSeq,_totalHashLength,_hash_bits,&(EcoHash::lessthan));
}
